@@ -14,12 +14,12 @@
 #include "cvs.h"
 #include "getline.h"
 
-static Node *AddEntryNode PROTO((List * list, Entnode *entnode));
+static Node *AddEntryNode (List * list, Entnode *entnode);
 
-static Entnode *fgetentent PROTO((FILE *, char *, int *));
-static int   fputentent PROTO((FILE *, Entnode *));
+static Entnode *fgetentent (FILE *, char *, int *);
+static int   fputentent (FILE *, Entnode *);
 
-static Entnode *subdir_record PROTO((int, const char *, const char *));
+static Entnode *subdir_record (int, const char *, const char *);
 
 static FILE *entfile;
 static char *entfilename;		/* for error messages */
@@ -29,21 +29,10 @@ static char *entfilename;		/* for error messages */
 /*
  * Construct an Entnode
  */
-static Entnode *Entnode_Create PROTO ((enum ent_type, const char *,
-				       const char *, const char *,
-				       const char *, const char *,
-				       const char *, const char *));
-
 static Entnode *
-Entnode_Create(type, user, vn, ts, options, tag, date, ts_conflict)
-    enum ent_type type;
-    const char *user;
-    const char *vn;
-    const char *ts;
-    const char *options;
-    const char *tag;
-    const char *date;
-    const char *ts_conflict;
+Entnode_Create (enum ent_type type, const char *user, const char *vn,
+                const char *ts, const char *options, const char *tag,
+                const char *date, const char *ts_conflict)
 {
     Entnode *ent;
     
@@ -64,11 +53,10 @@ Entnode_Create(type, user, vn, ts, options, tag, date, ts_conflict)
 /*
  * Destruct an Entnode
  */
-static void Entnode_Destroy PROTO ((Entnode *));
+static void Entnode_Destroy (Entnode *);
 
 static void
-Entnode_Destroy (ent)
-    Entnode *ent;
+Entnode_Destroy (Entnode *ent)
 {
     free (ent->user);
     free (ent->version);
@@ -86,11 +74,9 @@ Entnode_Destroy (ent)
 /*
  * Write out the line associated with a node of an entries file
  */
-static int write_ent_proc PROTO ((Node *, void *));
+static int write_ent_proc (Node *, void *);
 static int
-write_ent_proc (node, closure)
-     Node *node;
-     void *closure;
+write_ent_proc (Node *node, void *closure)
 {
     Entnode *entnode = node->data;
 
@@ -108,8 +94,7 @@ write_ent_proc (node, closure)
  * first of course
  */
 static void
-write_entries (list)
-    List *list;
+write_entries (List *list)
 {
     int sawdir;
 
@@ -168,15 +153,11 @@ write_entries (list)
  * Removes the argument file from the Entries file if necessary.
  */
 void
-Scratch_Entry (list, fname)
-    List *list;
-    const char *fname;
+Scratch_Entry (List *list, const char *fname)
 {
     Node *node;
 
-    if (trace)
-	(void) fprintf (stderr, "%s-> Scratch_Entry(%s)\n",
-			CLIENT_SERVER_STR, fname);
+    TRACE ( 1, "Scratch_Entry(%s)", fname );
 
     /* hashlookup to see if it is there */
     if ((node = findnode_fn (list, fname)) != NULL)
@@ -211,15 +192,9 @@ Scratch_Entry (list, fname)
  * removing the old entry first, if necessary.
  */
 void
-Register (list, fname, vn, ts, options, tag, date, ts_conflict)
-    List *list;
-    const char *fname;
-    const char *vn;
-    const char *ts;
-    const char *options;
-    const char *tag;
-    const char *date;
-    const char *ts_conflict;
+Register (List *list, const char *fname, const char *vn, const char *ts,
+          const char *options, const char *tag, const char *date,
+          const char *ts_conflict)
 {
     Entnode *entnode;
     Node *node;
@@ -231,14 +206,10 @@ Register (list, fname, vn, ts, options, tag, date, ts_conflict)
     }
 #endif
 
-    if (trace)
-    {
-	(void) fprintf (stderr, "%s-> Register(%s, %s, %s%s%s, %s, %s %s)\n",
-			CLIENT_SERVER_STR,
-			fname, vn, ts ? ts : "",
-			ts_conflict ? "+" : "", ts_conflict ? ts_conflict : "",
-			options, tag ? tag : "", date ? date : "");
-    }
+    TRACE ( 1, "Register(%s, %s, %s%s%s, %s, %s %s)",
+	    fname, vn, ts ? ts : "",
+	    ts_conflict ? "+" : "", ts_conflict ? ts_conflict : "",
+	    options, tag ? tag : "", date ? date : "" );
 
     entnode = Entnode_Create (ENT_FILE, fname, vn, ts, options, tag, date,
 			      ts_conflict);
@@ -271,8 +242,7 @@ Register (list, fname, vn, ts, options, tag, date, ts_conflict)
  * Node delete procedure for list-private sticky dir tag/date info
  */
 static void
-freesdt (p)
-    Node *p;
+freesdt (Node *p)
 {
     struct stickydirtag *sdtp = p->data;
 
@@ -287,10 +257,7 @@ freesdt (p)
    On error, prints an error message and returns NULL.  */
 
 static Entnode *
-fgetentent(fpin, cmd, sawdir)
-    FILE *fpin;
-    char *cmd;
-    int *sawdir;
+fgetentent(FILE *fpin, char *cmd, int *sawdir)
 {
     Entnode *ent;
     char *line;
@@ -410,9 +377,7 @@ fgetentent(fpin, cmd, sawdir)
 }
 
 static int
-fputentent(fp, p)
-    FILE *fp;
-    Entnode *p;
+fputentent(FILE *fp, Entnode *p)
 {
     switch (p->type)
     {
@@ -460,9 +425,7 @@ fputentent(fp, p)
    messages, or NULL if not known (that is, noone has gotten around
    to updating the caller to pass in the information).  */
 List *
-Entries_Open (aflag, update_dir)
-    int aflag;
-    char *update_dir;
+Entries_Open (int aflag, char *update_dir)
 {
     List *entries;
     struct stickydirtag *sdtp = NULL;
@@ -572,8 +535,7 @@ Entries_Open (aflag, update_dir)
 }
 
 void
-Entries_Close(list)
-    List *list;
+Entries_Close(List *list)
 {
     if (list)
     {
@@ -592,8 +554,7 @@ Entries_Close(list)
  * node
  */
 static void
-Entries_delproc (node)
-    Node *node;
+Entries_delproc (Node *node)
 {
     Entnode *p = node->data;
 
@@ -605,9 +566,7 @@ Entries_delproc (node)
  * list
  */
 static Node *
-AddEntryNode (list, entdata)
-    List *list;
-    Entnode *entdata;
+AddEntryNode (List *list, Entnode *entdata)
 {
     Node *p;
 
@@ -635,17 +594,42 @@ AddEntryNode (list, entdata)
     return (p);
 }
 
+
+
+/*
+ * Write out the CVS/Template file.
+ */
+void
+WriteTemplate (const char *update_dir, int xdotemplate, const char *repository)
+{
+#ifdef SERVER_SUPPORT
+    TRACE (1, "Write_Template (%s, %s)", update_dir, repository);
+
+    if (noexec)
+	return;
+
+    if (server_active && xdotemplate)
+    {
+	/* Clear the CVS/Template if supported to allow for the case
+	 * where the rcsinfo file no longer has an entry for this
+	 * directory.
+	 */
+	server_clear_template (update_dir, repository);
+	server_template (update_dir, repository);
+    }
+#endif
+
+    return;
+}
+
+
+
 /*
  * Write out/Clear the CVS/Tag file.
  */
 void
-WriteTag (dir, tag, date, nonbranch, update_dir, repository)
-    const char *dir;
-    const char *tag;
-    const char *date;
-    int nonbranch;
-    const char *update_dir;
-    const char *repository;
+WriteTag (const char *dir, const char *tag, const char *date, int nonbranch,
+          const char *update_dir, const char *repository)
 {
     FILE *fout;
     char *tmp;
@@ -713,10 +697,7 @@ WriteTag (dir, tag, date, nonbranch, update_dir, repository)
    If there is an error, print an error message, set *DATEP and *TAGP
    to NULL, and return.  */
 void
-ParseTag (tagp, datep, nonbranchp)
-    char **tagp;
-    char **datep;
-    int *nonbranchp;
+ParseTag (char **tagp, char **datep, int *nonbranchp)
 {
     FILE *fp;
 
@@ -795,8 +776,7 @@ ParseTag (tagp, datep, nonbranchp)
  */
 
 void
-Subdirs_Known (entries)
-     List *entries;
+Subdirs_Known (List *entries)
 {
     struct stickydirtag *sdtp = entries->list->data;
 
@@ -834,10 +814,7 @@ Subdirs_Known (entries)
 /* Record subdirectory information.  */
 
 static Entnode *
-subdir_record (cmd, parent, dir)
-     int cmd;
-     const char *parent;
-     const char *dir;
+subdir_record (int cmd, const char *parent, const char *dir)
 {
     Entnode *entnode;
 
@@ -914,10 +891,7 @@ subdir_record (cmd, parent, dir)
  */
 
 void
-Subdir_Register (entries, parent, dir)
-     List *entries;
-     const char *parent;
-     const char *dir;
+Subdir_Register (List *entries, const char *parent, const char *dir)
 {
     Entnode *entnode;
 
@@ -934,16 +908,15 @@ Subdir_Register (entries, parent, dir)
 	Entnode_Destroy (entnode);
 }
 
+
+
 /*
  * Record the removal of a subdirectory.  The arguments are the same
  * as for Subdir_Register.
  */
 
 void
-Subdir_Deregister (entries, parent, dir)
-     List *entries;
-     const char *parent;
-     const char *dir;
+Subdir_Deregister (List *entries, const char *parent, const char *dir)
 {
     Entnode *entnode;
 
@@ -998,16 +971,13 @@ enum base_walk {
     BASE_DEREGISTER
 };
 
-static void base_walk PROTO ((enum base_walk, struct file_info *, char **));
+static void base_walk (enum base_walk, struct file_info *, char **);
 
 /* Read through the lines in CVS/Baserev, taking the actions as documented
    for CODE.  */
 
 static void
-base_walk (code, finfo, rev)
-    enum base_walk code;
-    struct file_info *finfo;
-    char **rev;
+base_walk (enum base_walk code, struct file_info *finfo, char **rev)
 {
     FILE *fp;
     char *line;
@@ -1160,8 +1130,7 @@ base_walk (code, finfo, rev)
    or NULL if not listed.  */
 
 char *
-base_get (finfo)
-    struct file_info *finfo;
+base_get (struct file_info *finfo)
 {
     char *rev;
     base_walk (BASE_GET, finfo, &rev);
@@ -1171,9 +1140,7 @@ base_get (finfo)
 /* Set the revision for FILE to REV.  */
 
 void
-base_register (finfo, rev)
-    struct file_info *finfo;
-    char *rev;
+base_register (struct file_info *finfo, char *rev)
 {
     base_walk (BASE_REGISTER, finfo, &rev);
 }
@@ -1181,8 +1148,7 @@ base_register (finfo, rev)
 /* Remove FILE.  */
 
 void
-base_deregister (finfo)
-    struct file_info *finfo;
+base_deregister (struct file_info *finfo)
 {
     base_walk (BASE_DEREGISTER, finfo, NULL);
 }
