@@ -207,38 +207,6 @@ Parse_Info (const char *infofile, const char *repository, CALLPROC callproc,
 
 
 
-/* Print a warning and return false if P doesn't look like a string specifying
- * a boolean value.
- *
- * Sets *VAL to the parsed value when it is found to be valid.  *VAL will not
- * be altered when false is returned.
- */
-static bool
-readBool (const char *infopath, const char *option, const char *p, bool *val)
-{
-    TRACE (TRACE_FLOW, "readBool (%s, %s, %s)", infopath, option, p);
-    if (!strcasecmp (p, "no") || !strcasecmp (p, "false")
-        || !strcasecmp (p, "off") || !strcmp (p, "0"))
-    {
-	TRACE (TRACE_DATA, "Read %d for %s", *val, option);
-	*val = false;
-	return true;
-    }
-    else if (!strcasecmp (p, "yes") || !strcasecmp (p, "true")
-	     || !strcasecmp (p, "on") || !strcmp (p, "1"))
-    {
-	TRACE (TRACE_DATA, "Read %d for %s", *val, option);
-	*val = true;
-	return true;
-    }
-
-    error (0, 0, "%s: unrecognized value '%s' for `%s'",
-	   infopath, option, p);
-    return false;
-}
-
-
-
 /* Print a warning and return false if P doesn't look like a string specifying
  * something that can be converted into a size_t.
  *
